Package: Cepo
Title: Cepo for the identification of differentially stable genes
Version: 1.16.0
Authors@R: 
    c(
    person(given = "Hani Jieun",
           family = "Kim",
           role = c("aut", "cre"),
           email = "hani.kim127@gmail.com",
           comment = c(ORCID = "0000-0003-1844-3275")),
    person(given = "Kevin",
           family = "Wang",
           role = c("aut"),
           email = "kevinwangstats@gmail.com",
           comment = c(ORCID = "0000-0003-2615-6102"))
    )
Description: Defining the identity of a cell is fundamental to understand the heterogeneity of cells to various environmental signals and perturbations. We present Cepo, a new method to explore cell identities from single-cell RNA-sequencing data using differential stability as a new metric to define cell identity genes. Cepo computes cell-type specific gene statistics pertaining to differential stable gene expression.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: false
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
Imports: DelayedMatrixStats, DelayedArray, HDF5Array, S4Vectors,
        methods, SingleCellExperiment, SummarizedExperiment, ggplot2,
        rlang, grDevices, patchwork, reshape2, BiocParallel, stats,
        dplyr, purrr
biocViews: Classification, GeneExpression, SingleCell, Software,
        Sequencing, DifferentialExpression
Suggests: knitr, rmarkdown, BiocStyle, testthat, covr, UpSetR, scater,
        scMerge, fgsea, escape, pheatmap
VignetteBuilder: knitr
Depends: GSEABase, R (>= 4.1)
git_url: https://git.bioconductor.org/packages/Cepo
git_branch: RELEASE_3_22
git_last_commit: 14015a6
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:03:56 UTC; biocbuild
Author: Hani Jieun Kim [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1844-3275>),
  Kevin Wang [aut] (ORCID: <https://orcid.org/0000-0003-2615-6102>)
Maintainer: Hani Jieun Kim <hani.kim127@gmail.com>
