# Generated by roxygen2: do not edit by hand

export(Cepo)
export(plotDensities)
export(topGenes)
import(dplyr)
import(ggplot2)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(DelayedArray,cbind)
importFrom(DelayedMatrixStats,rowMeans2)
importFrom(DelayedMatrixStats,rowSds)
importFrom(DelayedMatrixStats,rowSums2)
importFrom(HDF5Array,HDF5Array)
importFrom(SingleCellExperiment,colData)
importFrom(SummarizedExperiment,assays)
importFrom(grDevices,rainbow)
importFrom(methods,is)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
