#' @title cellbench
#'
#' @description A single-cell RNA-seq dataset adapted from [sc_mixology](https://github.com/LuyiTian/sc_mixology)
#'
#' @usage data(cellbench)
#'
#' @source https://github.com/LuyiTian/sc_mixology
#'
#' @format An object of SingleCellExperiment class with 895 cells and 2001 genes.
#'
#' @name cellbench
#' @docType data
#'
"cellbench"

#' @title sce_pancreas
#'
#' @description A subsampled single-cell RNA-seq dataset 
#' 
#' @usage data(sce_pancreas)
#'
#' @format An object of SingleCellExperiment class with 528 cells and 1358 genes.
#'
#' @name sce_pancreas
#' @docType data
#'
"sce_pancreas"
